import boolean


class AND(boolean.AND):

    def __bool__(self, tags_for_check=None):
        for arg in self.args:
            if not arg.__bool__(tags_for_check):
                return False
        return True


class OR(boolean.OR):

    def __bool__(self, tags_for_check=None):
        for arg in self.args:
            if arg.__bool__(tags_for_check):
                return True
        return False


class NOT(boolean.NOT):

    def __bool__(self, tags_for_check=None):
        if self.args[0].__bool__(tags_for_check):
            return False
        return True


class Symbol(boolean.Symbol):

    def __bool__(self, tags_for_check=None):
        if self.obj in tags_for_check:
            return True
        return False


class FALSE(boolean.boolean._FALSE):

    def __bool__(self, tags_for_check=None):
        if not tags_for_check:
            return True
        return False


class TRUE(boolean.boolean._TRUE):

    def __bool__(self, tags_for_check=None):
        if tags_for_check:
            return True
        return False


algebra = boolean.BooleanAlgebra(AND_class=AND, OR_class=OR, NOT_class=NOT, Symbol_class=Symbol,
                                 FALSE_class=FALSE, TRUE_class=TRUE)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import
from django.core.management.base import BaseCommand
from django.db import models
from ttoolly.utils import prepare_file_for_tests
from django.apps import apps
get_models = apps.get_models


class Command(BaseCommand):

    help = "Create random files for all models, if not exists on file system"

    def add_arguments(self, parser):
        parser.add_argument('--show', dest='show', action="store_true", default=False,
                            help='Only show output, not generate files')

    def handle(self, *args, **kwargs):
        verbosity = int(kwargs.get('verbosity'))
        all_models = get_models(include_auto_created=True)
        models_with_files = {}
        if verbosity:
            print('Found %s models' % len(all_models))
        for model in all_models:
            fields = [f for f in model._meta.fields if isinstance(f, models.FileField)]
            if fields:
                if verbosity > 1:
                    print('In model %s.%s (%s) found file fields: %s' %
                          (model._meta.app_label, model.__name__, model._meta.verbose_name,
                           ', '.join(['%s (%s)' % (f.name, f.verbose_name) for f in fields])))
                models_with_files[model] = fields

        if kwargs.get('show'):
            return
        for model in models_with_files.keys():
            if verbosity > 1:
                print('\nGenerate files for model %s.%s (%s)' %
                      (model._meta.app_label, model.__name__, model._meta.verbose_name))
            for field in models_with_files[model]:
                if verbosity > 1:
                    print('  Generate files for field %s (%s)' % (field.name, field.verbose_name))
                prepare_file_for_tests(model, field.name, verbosity=verbosity)



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from ttoolly.utils import unicode_to_readable
from django.core import serializers
from django.apps import apps
import sys
get_model = apps.get_model


class Command(BaseCommand):

    help = ("Write to file the contents of the database as a fixture with "
            "readable unicode text\n"
            "Example: manage.py udumpdata accounts.User -f /path/to/file")
    args = 'appname.ModelName'

    def add_arguments(self, parser):
        parser.add_argument('labels', metavar='label', nargs='+', help='Models for dump')
        parser.add_argument('-f', '--file', dest='path_to_file', help='Specifies the output file path.', required=True)
        parser.add_argument('--pk', dest='pks', nargs='+', help='Specifies pks of objects')

    def handle(self, *args, **kwargs):
        path_to_file = kwargs.get('path_to_file')

        labels = kwargs.get('labels')
        if len(labels) > 1 and kwargs.get('pks'):
            sys.exit("Multiple models labels can't be used with --pk arguments")
        objects_list = []
        for label in labels:
            app_label, model_label = label.split('.')
            obj_model = get_model(app_label, model_label)

            qs = obj_model.objects.all()
            if kwargs.get('pks'):
                qs = qs.filter(pk__in=kwargs.get('pks'))
            objects_list.extend(qs)

            if 'PolymorphicModelBase' in [b.__name__ for b in obj_model.__class__.__bases__]:
                polymorphic_model = obj_model.polymorphic_ctype.field.model
                polymorphic_model = polymorphic_model if polymorphic_model != obj_model else obj_model
                base_objects_qs = polymorphic_model.base_objects.filter(
                    pk__in=qs.values_list('pk', flat=True))
                objects_list.extend(base_objects_qs)

        text = unicode_to_readable(serializers.serialize('json',
                                                         objects_list, indent=4,
                                                         use_natural_foreign_keys=True))

        with open(path_to_file, 'ab') as f:
            f.write(text.encode('utf-8'))
